package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BaseDataEnt;
import pps.core.base.entity.BaseDataView;
import pps.core.base.mapper.BaseDataMapper;
import pps.core.base.mapper.BaseDataViewMapper;
import pps.core.base.service.data.base_data.*;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XService
public class BaseDataService {
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseData(XContext context, CreateBaseDataInput input){
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        BaseDataEnt entity = new BaseDataEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseData(XContext context, UpdateBaseDataInput input){
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDataEnt::getId, input.getId());
        BaseDataEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseData(XContext context, DeleteBaseDataInput input){
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDataEnt::getId, input.getId());
        BaseDataEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBaseDataOutput> getBaseData(XContext context, GetBaseDataInput input){
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDataEnt::getId, input.getId());
        BaseDataEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseDataOutput output = new GetBaseDataOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseDataOutput> queryBaseData(XContext context, QueryBaseDataInput input){
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        QueryWrapper<BaseDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseDataEnt::getOuId, input.getOuId())
            .eq(BaseDataEnt::getLineId, input.getLineId())
            .eq(BaseDataEnt::getDataDate, input.getDataDate());
        return XMapperHelper.query(mapper, input, queryWrapper, QueryBaseDataOutput.class);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBaseDataOutput> dynamicQueryBaseData(XContext context, CustomQueryInput input){
        BaseDataMapper mapper = context.getBean(BaseDataMapper.class);
        return XMapperHelper.query(mapper, input, BaseDataEnt.class, QueryBaseDataOutput.class);
    }
}
