package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.lang3.StringUtils;
import pps.core.base.entity.BaseLineAllView;
import pps.core.base.entity.BaseLineEnt;
import pps.core.base.entity.BaseLineView;
import pps.core.base.mapper.BaseLineMapper;
import pps.core.base.mapper.BaseLineViewMapper;
import pps.core.base.service.data.base_line.*;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.data.CustomQueryInput;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.util.List;

@XService
public class BaseLineService {
    @XApiAnonymous
    @XApiPost
    public XServiceResult createBaseLine(XContext context, CreateBaseLineInput input){
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        BaseLineEnt entity = new BaseLineEnt();
        XCopyUtils.copyObject(input, entity);
        mapper.insert(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBaseLine(XContext context, UpdateBaseLineInput input){
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        QueryWrapper<BaseLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseLineEnt::getId, input.getId());
        BaseLineEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBaseLine(XContext context, DeleteBaseLineInput input){
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        QueryWrapper<BaseLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseLineEnt::getId, input.getId());
        BaseLineEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiPost
    public XSingleResult<GetBaseLineOutput> getBaseLine(XContext context, GetBaseLineInput input){
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        QueryWrapper<BaseLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BaseLineEnt::getId, input.getId());
        BaseLineEnt entity = mapper.selectOne(queryWrapper);
        if(entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseLineOutput output = new GetBaseLineOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiPost
    public XListResult<QueryBaseLineOutput> queryBaseLineList(XContext context, QueryBaseLineListInput input){
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        List<BaseLineEnt> list = mapper.selectList(new QueryWrapper<>());
        List<QueryBaseLineOutput> outputs = XCopyUtils.copyNewList(list, QueryBaseLineOutput.class);
        return XListResult.success(outputs);
    }

    @XApiAnonymous
    @XApiPost
    public XPageResult<QueryBaseLineOutput> queryBaseLine(XContext context, QueryBaseLineInput input){
        BaseLineViewMapper mapper = context.getBean(BaseLineViewMapper.class);
        BaseLineView view = XCopyUtils.copyNewObject(input,BaseLineView.class);
        List<BaseLineView> list = mapper.selectList(view);
        PageInfo<BaseLineView> pageInfo = new PageInfo(list);
        List<QueryBaseLineOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseLineOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiAnonymous
    @XApiPost
    public XPageResult<QueryBaseLineAllOutput> queryAllBaseLine(XContext context, QueryBaseLineInput input){
        BaseLineViewMapper mapper = context.getBean(BaseLineViewMapper.class);
        BaseLineView view = XCopyUtils.copyNewObject(input,BaseLineView.class);
        List<BaseLineAllView> list = mapper.selectDataList(view);
        PageInfo<BaseLineAllView> pageInfo = new PageInfo(list);
        List<QueryBaseLineAllOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBaseLineAllOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XApiAnonymous
    @XApiPost
    public XPageResult<QueryBaseLineOutput> dynamicQueryBaseLine(XContext context, CustomQueryInput input){
        BaseLineMapper mapper = context.getBean(BaseLineMapper.class);
        return XMapperHelper.query(mapper, input, BaseLineEnt.class, QueryBaseLineOutput.class);
    }

}
